@@ -45,7 +45,7 @@ def bio_rates(xpar=default_Xbulk, spar=default_Sbulk, aob=kinetics.aob, nob=kine
     #[0 AOB_growth, 1 AOB_decay, 2 NOB_growth, 3 NOB_decay, 4 AMX_growth, 5 AMX_decay, 6 CMX_growth, 7 CMX_decay, 8 OHO_growth_O2, 9 OHO_growth_NO2, 10 OHO_growth_NO3, 11 OHO_decay])
 
 class Reactor:
-    def __init__(self, influent='default', volume=4000, VSS=3000, DO=2, Xbulk=default_Xbulk, Sbulk=default_Sbulk, area=100, Lmax=400*(10**-6), Lbf=5*(10**-6), VSmax=25000, DBL=10*(10**-6)):
+    def __init__(self, influent='default', volume=4000, VSS=3000, DO=2, Xbulk=default_Xbulk, Sbulk='influent', area=100, Lmax=400*(10**-6), Lbf=5*(10**-6), VSmax=25000, DBL=10*(10**-6)):
         if influent == 'default':
             inf = load_default_influent()
             for p in xparlist+sparlist:
@@ -73,9 +73,16 @@ class Reactor:
         self.volumes = [volume]
         self.VSS = [VSS]
         self.DO = [DO]
-        Sbulk[0] = DO
         self.Xbulk = [VSS*Xbulk/(Xbulk.sum())]
-        self.Sbulk = [Sbulk]
+        if Sbulk == 'influent':
+            sbulkarr = np.zeros(len(sparlist))
+            for i, s in enumerate(sparlist):
+                sbulkarr[i] = self.influents[-1].iloc[0][s]
+            self.Sbulk = [sbulkarr]
+        else:
+            self.Sbulk = [Sbulk]
+        self.Sbulk[-1][0] = DO
+        
         self.areas = [area]
         self.Lmax = [Lmax]
         self.Lbf = [Lbf]
@@ -84,7 +91,7 @@ class Reactor:
         bf_x = VSmax*Xbulk/Xbulk.sum()
         bf_x = np.tile(bf_x, (BF_layers+1, 1))
         bf_x[-1] = np.zeros(len(bf_x[-1]))
-        bf_s = np.tile(Sbulk, (BF_layers+1, 1))
+        bf_s = np.tile(self.Sbulk[-1], (BF_layers+1, 1))
         self.Xbiofilm = [bf_x]
         self.Sbiofilm = [bf_s]
         self.recircT = {}
@@ -314,7 +321,7 @@ class Reactor:
 
                             b = self.Sbiofilm[i][:, s].copy()
                             b = 4*b - Sbf_prev[i][:, s]
-                            b[:-1] = b[:-1] + 2*dts*(2*bf_brS-bf_brS1)
+                            b[:-1] = b[:-1] + 2*dts*(2*bf_brS-bf_brS1) #Add reaction term
  
                             new_Sbiofilm[:, s] = scipy.sparse.linalg.spsolve(A_matrix_bf[i][s], b)
                             new_Sbiofilm[-1, s] = self.Sbulk[i][s]
@@ -324,7 +331,7 @@ class Reactor:
                             if change_conc_profile > Sbiofilm_change:
                                 Sbiofilm_change = change_conc_profile
 
-                            #Calculate biofilm contribution to bulk removal of s
+                            #Calculate biofilm contribution to bulk removal of S in g/m3.d reactor volume
                             delta_s[s] = sum(bf_brS)*self.Lbf[i]*self.areas[i]/self.volumes[i]
 
                         Sbf_prev[i] = self.Sbiofilm[i].copy()
@@ -377,6 +384,9 @@ class Reactor:
                         brS = brS.sum(axis=0)
                         brS1 = bulk_br_prev[i]*Spm
                         brS1 = brS1.sum(axis=0)
+                    else:
+                        brS = np.zeros(len(sparlist))
+                        brS1 = np.zeros(len(sparlist))
 
                     #Get new concentrations in current volume
                     new_Sbulk = (4*self.Sbulk[i]-Sbulk_prev[i]+2*dts*(2*brS-brS1)+2*dts*delta_s)/(3+2*Qin_total[i]*dts/self.volumes[i])
